@@ -1,6 +1,6 @@
 import re
-from typing import List, Tuple, Union
-from urllib.parse import parse_qsl, urlparse
+from typing import Callable, List, Tuple, Union
+from urllib.parse import parse_qsl, urlparse, urlsplit, urlunsplit
 
 import requests
 from bs4 import BeautifulSoup
@@ -10,7 +10,7 @@ from w3lib.url import url_query_cleaner
 __author__ = "Elton H.Y. Chou"
 
 __license__ = "MIT"
-__version__ = "0.0.5"
+__version__ = "0.0.6"
 __maintainer__ = "Elton H.Y. Chou"
 __email__ = "plscd748@gmail.com"
 
@@ -51,6 +51,9 @@ class NoMoreQS:
         url : str
             Any useable url.
 
+        cookies : dict, optional
+            cookies for request
+
         Returns
         -------
         str
@@ -58,7 +61,7 @@ class NoMoreQS:
         """
         fld = get_fld(url)
 
-        cleaner = _super_cleaner if self.strict else _fbclid_cleaner
+        cleaner: Callable = _super_cleaner if self.strict else _fbclid_cleaner
 
         is_allowed_fld = fld in self.exclude_flds
         if is_allowed_fld:
@@ -99,14 +102,18 @@ def _super_cleaner(url: str, headers: dict = {}, cookies: dict = {}) -> str:
     url : str
         Any useable url.
 
-    kwargs : dict, optional
-        Optional arguments that ``request`` takes.
+    headers : dict, optional
+        Optional headers ``request`` takes.
+
+    cookeis : dict, optional
+        Optional cookies ``request takes.
 
     Returns
     -------
     str
         cleaned url, fbclid is always be cleaned.
     """
+    fragment = urlparse(url).fragment
     url = _fbclid_cleaner(url)
     page = _get_page(url, headers, cookies)
 
@@ -124,7 +131,7 @@ def _super_cleaner(url: str, headers: dict = {}, cookies: dict = {}) -> str:
     canonical_qs_len = count_qs_length(canonical_url)
     og_qs_len = count_qs_length(og_url)
 
-    # path_len > qs_len > -(netloc)
+    # Order weights: path_len -> qs_len -> -(netloc)
     candidate_urls = sorted([
         (origin_path_len, origin_qs_len, -len(urlparse(url).netloc), url),
         (canonical_path_len, canonical_qs_len, -len(urlparse(canonical_url).netloc), canonical_url),
@@ -133,6 +140,10 @@ def _super_cleaner(url: str, headers: dict = {}, cookies: dict = {}) -> str:
 
     for path_len, _, _, the_url in candidate_urls:
         if path_len:
+            if fragment:
+                url_components = urlsplit(the_url)
+                url_components = url_components._replace(fragment=fragment)
+                the_url = urlunsplit(url_components)
             return the_url
 
     return url
@@ -152,7 +163,9 @@ def _fbclid_cleaner(url: str, **kwargs) -> str:
     str
         cleaned url, fbclid is always be cleaned.
     """
-    url = url_query_cleaner(url, ("fbclid"), remove=True)
+    url = url_query_cleaner(url, ("fbclid"), remove=True, keep_fragments=True)
+    if url.endswith("#"):
+        return url[:-1]
     return url
 
 
@@ -235,7 +248,7 @@ def _get_page(url: str, headers: dict = {}, cookies: dict = {}) -> BeautifulSoup
     return page
 
 
-def parse_url_qs_to_dict(url: str) -> dict:
+def parse_url_qs_to_dict(url: str, as_set=False) -> Union[dict, set]:
     """
     Return qs as dict, if no qs return {}
 
@@ -244,10 +257,12 @@ def parse_url_qs_to_dict(url: str) -> dict:
     url : str
         validate url
 
+    as_set : bool, optional
+        use ``set`` as return type
+
     Returns
     -------
-    dict
-        qs as dict
+    Union[dict, set]
     """
     if not url:
         return {}
@@ -255,8 +270,30 @@ def parse_url_qs_to_dict(url: str) -> dict:
     qs = urlparse(url).query
     dict_qs = dict(parse_qsl(qs))
 
-    return dict_qs
+    return set(dict_qs) if as_set else dict_qs
 
 
 def count_qs_length(url: str) -> int:
+    """query string counting"""
     return len(parse_url_qs_to_dict(url)) if url else 0
+
+
+def qs_delta(original_url: str, cleaned_url: str) -> set:
+    """
+    query string delta as set
+
+    Parameters
+    ----------
+    original_url : str
+
+    cleaned_url : str
+
+    Returns
+    -------
+    set
+        set query string delta
+    """
+    original_qs = parse_url_qs_to_dict(original_url, as_set=True)
+    cleaned_qs = parse_url_qs_to_dict(cleaned_url, as_set=True)
+
+    return original_qs - cleaned_qs
